"""Enable CLI."""
import asyncio
import json
from typing import List

import click

from aioazuredevops.client import DevOpsClient
from aioazuredevops.core import DevOpsProject
from aioazuredevops.builds import DevOpsBuild


@click.command()
@click.option("--organization", "-u", help="Organization")
@click.option("--project", "-p", help="Project")
@click.option("--pat", "-t", help="Personal Access Token")
def cli(organization: str, project: str, pat: str = None):
    """CLI for this package."""
    asyncio.run(handle(organization, project, pat))


async def handle(organization: str, project: str, pat: str = None) -> None:
    client = DevOpsClient()
    if pat is not None:
        if await client.authorize(pat, organization) is True:
            print("Authenticated.")
        else:
            return
    print("Project:")
    doProject: DevOpsProject = await client.get_project(organization, project)
    if doProject is not None:
        print(doProject.id)
        print(doProject.name)
        print(doProject.description)
    print("Builds:")
    builds: List[DevOpsBuild] = await client.get_builds(
        organization,
        project,
        "?queryOrder=queueTimeDescending&maxBuildsPerDefinition=1",
    )
    if builds is not None and len(builds) > 0:
        build: DevOpsBuild = builds[0]
        if build is not None:
            print(build.id)
            print(build.build_number)
            print(build.status)
            print(build.result)
            print(build.source_branch)
            print(build.source_version)
            print(build.definition.id)
            print(build.definition.name)
            print(build.project.id)
            print(build.project.name)


cli()  # pylint: disable=E1120



"""DevOps Core. https://docs.microsoft.com/en-gb/rest/api/azure/devops/core/?view=azure-devops-rest-5.1"""
from datetime import datetime
from typing import List


class DevOpsLinks:
    """Links."""

    def __init__(self, linkSelf: str, linkCollection: str, linkWeb: str):
        """Initialize"""
        self.self = linkSelf
        self.collection = linkCollection
        self.web = linkWeb


class DevOpsTeam:
    """DevOps Team."""

    def __init__(self, id: str, name: str, url: str):
        """Initialize"""
        self.id = id
        self.name = name
        self.url = url


class DevOpsProject:
    """A DevOps Project"""

    def __init__(
        self,
        id: str,
        name: str,
        description: str,
        url: str,
        state: str,
        revision: int,
        visibility: str,
        last_updated: datetime,
        default_team: DevOpsTeam = None,
        links: DevOpsLinks = None,
    ):
        """Initialize"""
        self.id = id
        self.name = name
        self.description = description
        self.url = url
        self.state = state
        self.revision = revision
        self.visibility = visibility
        self.last_updated = last_updated
        self.default_team = default_team
        self.links = links



"""DevOps Builds. https://docs.microsoft.com/en-gb/rest/api/azure/devops/build/builds?view=azure-devops-rest-5.1"""
from datetime import datetime

from aioazuredevops.core import DevOpsProject


class DevOpsBuildLinks:
    """DevOps build links."""

    def __init__(
        self,
        lSelf: str,
        web: str,
        source_version_display_uri: str,
        timeline: str,
        badge: str,
    ):
        """Initialize"""
        self.self = lSelf
        self.web = web
        self.source_version_display_uri = source_version_display_uri
        self.timeline = timeline
        self.badge = badge


class DevOpsBuildDefinition:
    """DevOps build definition."""

    def __init__(
        self,
        id: int,
        name: str,
        url: str,
        path: str,
        type: str,
        queue_status: str,
        revision: int,
    ):
        self.id = id
        self.name = name
        self.url = url
        self.path = path
        self.type = type
        self.queue_status = queue_status
        self.revision = revision


class DevOpsBuild:
    """DevOps build."""

    def __init__(
        self,
        id: int,
        links: DevOpsBuildLinks,
        build_number: str,
        status: str,
        result: str,
        source_branch: str,
        source_version: str,
        priority: str,
        reason: str,
        queue_time: str,
        start_time: str,
        finish_time: str,
        definition: DevOpsBuildDefinition,
        project: DevOpsProject,
    ):
        """Initialize"""
        self.id = id
        self.links = links
        self.build_number = build_number
        self.status = status
        self.result = result
        self.source_branch = source_branch
        self.source_version = source_version
        self.priority = priority
        self.reason = reason
        self.queue_time = queue_time
        self.start_time = start_time
        self.finish_time = finish_time
        self.definition = definition
        self.project = project



"""Get data from the Azure DevOps API."""
import asyncio
import json
import sys
from datetime import datetime, timedelta
from typing import List

import aiohttp

from aioazuredevops.core import DevOpsLinks, DevOpsProject, DevOpsTeam
from aioazuredevops.builds import DevOpsBuild, DevOpsBuildDefinition, DevOpsBuildLinks


class DevOpsClient:
    """Client for Azure DevOps."""

    def __init__(self) -> None:
        """Initilalize."""
        self.pat = None

    async def fetch(
        self, session: aiohttp.ClientSession, url: str
    ) -> aiohttp.ClientResponse:
        """Runs a GET request and returns response"""
        if self.pat is None:
            return await session.get(url)
        else:
            return await session.get(
                url, headers={"Authorization": aiohttp.BasicAuth("", self.pat).encode()}
            )

    async def authorize(self, pat: str, organization: str) -> bool:
        """Authenticate."""
        async with aiohttp.ClientSession() as session:
            self.pat = pat
            response: aiohttp.ClientResponse = await self.fetch(
                session, f"https://dev.azure.com/{organization}/_apis/projects"
            )
            if response.status is 200:
                return True
            return False

    async def get_project(self, organization: str, project: str) -> DevOpsProject:
        """Get DevOps project."""
        async with aiohttp.ClientSession() as session:
            response: aiohttp.ClientResponse = await self.fetch(
                session,
                f"https://dev.azure.com/{organization}/_apis/projects/{project}",
            )
            if response.status is not 200:
                return None
            json = await response.json()
            if json is None:
                return None

            return DevOpsProject(
                json["id"],
                json["name"],
                json["description"],
                json["url"],
                json["state"],
                json["revision"],
                json["visibility"],
                datetime.strptime(json["lastUpdateTime"], "%Y-%m-%dT%H:%M:%S.%fZ"),
                DevOpsTeam(
                    json["defaultTeam"]["id"],
                    json["defaultTeam"]["name"],
                    json["defaultTeam"]["url"],
                ),
                DevOpsLinks(
                    json["_links"]["self"]["href"],
                    json["_links"]["collection"]["href"],
                    json["_links"]["web"]["href"],
                ),
            )

    async def get_builds(
        self, organization: str, project: str, parameters: str
    ) -> List[DevOpsBuild]:
        """Get DevOps builds."""
        async with aiohttp.ClientSession() as session:
            response: aiohttp.ClientResponse = await self.fetch(
                session,
                f"https://dev.azure.com/{organization}/{project}/_apis/build/builds{parameters}",
            )
            if response.status is not 200:
                return None
            json = await response.json()
            if json is None:
                return None

            builds = []
            for build in json["value"]:
                builds.append(
                    DevOpsBuild(
                        build["id"],
                        DevOpsBuildLinks(
                            build["_links"]["self"]["href"],
                            build["_links"]["web"]["href"],
                            build["_links"]["sourceVersionDisplayUri"]["href"],
                            build["_links"]["timeline"]["href"],
                            build["_links"]["badge"]["href"],
                        ),
                        build["buildNumber"],
                        build["status"],
                        build["result"],
                        build["sourceBranch"],
                        build["sourceVersion"],
                        build["priority"],
                        build["reason"],
                        build["queueTime"],
                        build["startTime"],
                        build["finishTime"],
                        DevOpsBuildDefinition(
                            build["definition"]["id"],
                            build["definition"]["name"],
                            build["definition"]["url"],
                            build["definition"]["path"],
                            build["definition"]["type"],
                            build["definition"]["queueStatus"],
                            build["definition"]["revision"],
                        ),
                        DevOpsProject(
                            build["project"]["id"],
                            build["project"]["name"],
                            build["project"]["description"],
                            build["project"]["url"],
                            build["project"]["state"],
                            build["project"]["revision"],
                            build["project"]["visibility"],
                            datetime.strptime(
                                build["project"]["lastUpdateTime"],
                                "%Y-%m-%dT%H:%M:%S.%fZ",
                            ),
                        ),
                    )
                )

            return builds

    async def get_build(
        self, organization: str, project: str, parameters: str, build_id: int
    ) -> DevOpsBuild:
        """Get DevOps builds."""
        async with aiohttp.ClientSession() as session:
            response: aiohttp.ClientResponse = await self.fetch(
                session,
                f"https://dev.azure.com/{organization}/{project}/_apis/build/builds/{build_id}",
            )
            if response.status is not 200:
                return None
            build = await response.json()
            if build is None:
                return None

            return DevOpsBuild(
                build["id"],
                DevOpsBuildLinks(
                    build["_links"]["self"]["href"],
                    build["_links"]["web"]["href"],
                    build["_links"]["sourceVersionDisplayUri"]["href"],
                    build["_links"]["timeline"]["href"],
                    build["_links"]["badge"]["href"],
                ),
                build["buildNumber"],
                build["status"],
                build["result"],
                build["sourceBranch"],
                build["sourceVersion"],
                build["priority"],
                build["reason"],
                build["queueTime"],
                build["startTime"],
                build["finishTime"],
                DevOpsBuildDefinition(
                    build["definition"]["id"],
                    build["definition"]["name"],
                    build["definition"]["url"],
                    build["definition"]["path"],
                    build["definition"]["type"],
                    build["definition"]["queueStatus"],
                    build["definition"]["revision"],
                ),
                DevOpsProject(
                    build["project"]["id"],
                    build["project"]["name"],
                    build["project"]["description"],
                    build["project"]["url"],
                    build["project"]["state"],
                    build["project"]["revision"],
                    build["project"]["visibility"],
                    datetime.strptime(
                        build["project"]["lastUpdateTime"], "%Y-%m-%dT%H:%M:%S.%fZ",
                    ),
                ),
            )



"""Setup configuration."""
import setuptools

with open("README.md", "r") as fh:
    LONG = fh.read()

setuptools.setup(
    name="aioazuredevops",
    version="1.3.0",
    author="Timmo",
    author_email="contact@timmo.xyz",
    description="Get data from the Azure DevOps API.",
    long_description=LONG,
    long_description_content_type="text/markdown",
    install_requires=[
        'aiohttp==3.6.2',
        'click==7.1.2'
    ],
    entry_points={
        'console_scripts': [
            'aioazuredevops = aioazuredevops.cli:cli'
        ]
    },
    url="https://github.com/timmo001/aioazuredevops",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    )
)
